"""
Multi-Agent System for Enhanced Project Generation.

This package provides AutoGen-powered multi-agent collaboration for generating
high-quality projects with specialized expertise in different areas.
"""

from .multi_agent_system import MultiAgentProjectGenerator

__all__ = ["MultiAgentProjectGenerator"]